%{

/*
 * action-lex.l
 * 
 * A lexer for the commands for the `wimupdate' program.
 *
 * wimlib - Library for working with WIM files 
 *
 * Copyright (C) 2012 Eric Biggers
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option) any
 * later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include "wimupdate.h"
#include "action.h"
#include <string.h>

#define do_action(CMD)  \
do { \
	yylval.command_word = CMD;  \
	return TOK_COMMAND_WORD; \
} while (0)

%}

%option noyywrap

%x dqstring
%x sqstring
%x command
%x shell

%%

\!		{ BEGIN(shell); do_action(SHELL); }
abort		{ BEGIN(command); do_action(ABORT); }
add		{ BEGIN(command); do_action(ADD); }
cat		{ BEGIN(command); do_action(CAT); }
cd		{ BEGIN(command); do_action(CD); }
exit		{ BEGIN(command); do_action(EXIT); }
help		{ BEGIN(command); do_action(HELP); }
image		{ BEGIN(command); do_action(IMAGE); }
\?		{ BEGIN(command); do_action(HELP); }
ls		{ BEGIN(command); do_action(LS); }
mkdir		{ BEGIN(command); do_action(MKDIR); }
mv		{ BEGIN(command); do_action(MV); }
rm		{ BEGIN(command); do_action(RM); }
shell		{ BEGIN(shell); do_action(SHELL); }
show		{ BEGIN(command); do_action(SHOW); }
stat		{ BEGIN(command); do_action(STAT); }
write		{ BEGIN(command); do_action(WRITE); }
<command>;		{ BEGIN(INITIAL); return TOK_END_OF_COMMAND; }
<command>\n		{ BEGIN(INITIAL); return TOK_END_OF_COMMAND; }
<command>\"		{ BEGIN(dqstring); }
<command>\'		{ BEGIN(sqstring); }
<command>[^ '"\t\n;]+ 	{ yylval.str = xstrdup(yytext); return TOK_ARG; }
<sqstring>[^']*		{ printf(yytext); yylval.str = xstrdup(yytext); return TOK_ARG; }
<dqstring>[^"]*		{ yylval.str = xstrdup(yytext); return TOK_ARG; }
<sqstring>\'		{ BEGIN(command); }
<dqstring>\"		{ BEGIN(command); }
<shell>[^\n]+		{ BEGIN(command); yylval.str = xstrdup(yytext); return TOK_ARG;}
.		{ }

%%


/* Executes the actions given by the NULL-terminated string in @s.  */
bool execute_action_string(const char *s)
{
	void *buf = yy_scan_string(s);
	if (yyparse() != 0)
		return false;
	yy_delete_buffer(buf);

	return true;
}
