/*
 * wimlib.h
 *
 * External header for wimlib.
 *
 * wimlib - Library for working with WIM files 
 *
 * Copyright (C) 2010 Carl Thijssen
 * Copyright (C) 2012 Eric Biggers
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option) any
 * later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#ifndef _WIMLIB_H
#define _WIMLIB_H

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>

#ifndef _WIMLIB_INTERNAL_H
typedef struct WIMStruct WIMStruct;
#endif

WIMStruct *wimlib_create();
bool wimlib_begin_read(WIMStruct *w, const char *filename);
bool wimlib_select_image(WIMStruct *w, unsigned image);
unsigned wimlib_get_num_images(const WIMStruct *w);
unsigned wimlib_get_current_image(const WIMStruct *w);
char *wimlib_get_working_directory_fullpath(WIMStruct *w);
char *wimlib_get_filename(WIMStruct *w);
bool wimlib_all_images_written(WIMStruct *w);
void wimlib_set_no_error_messages();
void wimlib_set_print_files(WIMStruct *w);
void wimlib_set_link_type(WIMStruct *w, int link_type);
bool wimlib_set_output_dir(WIMStruct *w, const char *dir);

bool wimlib_print_wim_header(WIMStruct *w);
bool wimlib_print_lookup_table(WIMStruct *w);
bool wimlib_print_xml_data(WIMStruct *w);
bool wimlib_print_boot_metadata(WIMStruct *w);
bool wimlib_print_files(WIMStruct *w);

bool wimlib_extract_tree(WIMStruct *w, const char *path);
bool wimlib_extract_image(WIMStruct *w);
bool wimlib_extract_all_images(WIMStruct *w);
bool wimlib_extract_xml_data(WIMStruct *w, const char *out_file);

bool wimlib_is_directory(WIMStruct *w, const char *path);

bool wimlib_add(WIMStruct *w, const char *source, const char *dest);
bool wimlib_cat(WIMStruct *w, const char *path);
bool wimlib_cd(WIMStruct *w, const char *working_directory);
bool wimlib_ls(WIMStruct *w, const char *directory, bool recursive);
bool wimlib_mkdir(WIMStruct *w, const char *directory, bool parents);
bool wimlib_mv(WIMStruct *w, const char *source, const char *dest);
bool wimlib_rm(WIMStruct *w, const char *file, bool recursive, bool verbose);
bool wimlib_stat(WIMStruct *w, const char *path);
bool wimlib_write(WIMStruct *w, const char *out_wim_path, bool current_image_only);

FILE *wimlib_fopen_and_get_length(const char *path, const char *mode, 
							off_t *len_ret);
void *wimlib_file_get_contents(const char *name, off_t *len_ret);

WIMStruct *wimlib_create_new_wim(unsigned flags, unsigned boot_idx);
bool wimlib_add_image(WIMStruct *w, const char *path);

void wimlib_end_read(WIMStruct *w);
void wimlib_destroy(WIMStruct *w);


#define WIM_LINK_TYPE_HARD     0
#define WIM_LINK_TYPE_SYMBOLIC 1
#define WIM_LINK_TYPE_NONE     2

#define WIM_FLAG_UNCOMPRESSED  0
#define WIM_FLAG_LZX           1


#endif

